/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: Factory.java,v 1.4 2003/07/18 10:08:05 plasmoid Exp $ 
 */
package org.thc.shagg;
import  org.thc.shagg.algorithms.*;

import java.util.Vector;

/**
 * This class is a factory that generates serial numbers. Information 
 * necessary are obtained from the supplied states file. Additionally
 * a prefix can be specified to obtain more "natural" numbers.
 * 
 * @version $Id: Factory.java,v 1.4 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */  
public class Factory
{
   private State[] states;
   private int numbers, length;
   private CheckDigitAlgorithm[] algorithms;
   private int[] prefix;
   
   
   /**
    * Construct a factoy. 
    * Read in the states file and generate the requested amount of 
    * serial numbers.
    * @param filename Filename of states
    * @param num Amount of serial numbers to generate
    */
   public Factory(String filename) {
      Vector results=null;

      try {
         results=State.loadVector(filename);
      } catch (Exception e) {
         Shagg.abort("Could not load states, "+filename+" is invalid.");
      }

      if(results.size()<=0)
         Shagg.abort("No states found in "+filename);

      states = new State[results.size()];
      for(int i=0;i<results.size();i++)
         states[i]=(State)results.get(i);

      numbers=Shagg.numbersToGenerate;
      prefix=Shagg.numberPrefix;
      length=states[0].getLength();
      algorithms=Shagg.initAlgorithms();
   }

   /**
    * Generate serial numbers. 
    * This is currently a rather simple method. No real intelligence. 
    * The code generate random numbers and checks if they match the 
    * different states read from the filename provided to the construcor.
    * If all states match, the numbers is printed to STDOUT. 
    */
   public void work() {
      int[] number;
      boolean match;

      for(int i=0;i<numbers;) {
         number=randomNumber();    

         match=true;
         for(int j=0;match && j<states.length;j++) 
            match&=states[j].check(number,algorithms);         

         if(match) {
            for(int j=0;j<length;j++)
               System.out.print(number[j]);
            System.out.println();   
            i++;
         }
      }   
   }

   /**
    * Generate a random number.
    * @param length of number
    * @return number as an array
    */
   public int[] randomNumber() {
      int i;
      int[] number=new int[length];

      for(i=0;i<prefix.length;i++)
         number[i]=prefix[i];

      for(;i<length;i++)
         number[i]=(int)(Math.random()*Shagg.digitRadix);
      
      return number;   
   }        
}
