/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: WeightSet.java,v 1.6 2003/07/18 10:08:05 plasmoid Exp $ 
 */
package org.thc.shagg;

import java.util.Vector;

/**
 * This class holds a set of weights. Weights are a sequence of numbers, 
 * using the provided radix it is possible to also generate weights for
 * non-digit serial numbers. 
 * <p>
 * The set contains all sequences until the specified length, weights that
 * only contain zeros are omitted. 
 * <p>
 * e.g. radix 3, length 2:<ul><tt>
 *    &nbsp;1,&nbsp;2<br>
 *    01,02,10,11,12,20,21,22<br>
 * </tt></ul>   
 *
 * @version $Id: WeightSet.java,v 1.6 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */  
public class WeightSet 
{
   private Vector set;
   
   /** 
    * Construct a weight set. 
    * @param radix Radix of numbers
    * @param len Maxmimum length of weight sequences
    */
   public WeightSet(int radix, int len) {
      int[] weights;
      set=new Vector();
      
      for(int l=1;l<=len;l++) {
      
         for(int i=0;i<(int)Math.pow(radix,l);i++) {
            weights=new int[l];         
            int zero=0;
     
            /*
             * This is ugly mathematical stuff. Instead of using strange
             * string manipulations, we manually extract the n-th position
             * from the integer i by using modulus and division. Tricky?
             */ 
            for(int j=0;j<l;j++) {
               weights[j]=i%(int)Math.pow(radix,l-j);
               weights[j]-=i%(int)Math.pow(radix,l-j-1);                 
               weights[j]/=(int)Math.pow(radix,l-j-1);
               
               if(weights[j]==0)
                  zero++;
            }
      
            if(zero==0)
               set.add(weights);                
         }   
      }
   }

   /**
    * Get the size of the set.
    * @return size Size of the weight set.
    */
   public int size() {
      return set.size();
   }
   
   /**
    * Get the i-th element of the set.
    * @param i Index of the element
    * @return Weights as an array of integers
    */
   public int[] get(int i) {
      return (int[]) set.get(i);
   }
}
