/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: CheckDigitAlgorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $ 
 */
package org.thc.shagg.algorithms;

/**
 * Basic method definitions for a check digit algorithm.  
 * The last digit of a supplied sequence of digits is verified using the
 * provided check() method. If the algorithm uses weights the optional
 * weights argument is used, otherwise it is set to null.
 *
 * @version $Id: CheckDigitAlgorithm.java,v 1.7 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */
public interface CheckDigitAlgorithm 
{
   /**
    * Checks if the supplied sequence matches the algorithm. (Optional)
    * using the supplied weights.
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @param weights  Weight sequence that can be of any length. If the
    *                 algorithm doesn't uses weights, null should be 
    *                 supplied.
    * @return wether sequences matches the algorithm.
    */
   public boolean check(int[] sequence, int[] weights);
   
   /**
    * Checks if the algorithm uses weights for the check digit calculation.
    * @return true if the algorithm uses weights otherwise false.
    */
   public boolean usesWeights();
   
   /**
    * Get the name of the implemented check digit algorithm.
    * @return algorithm name
    */
   public String getName();
   
   /**
    * Get the symbol of the implemented check digit algorithm. A short
    * synonym should be used to represent the algorithm during the 
    * analyse progress, e.g. "10-L" for a Modulus-10 Luhn algorithm,
    * @return algorithm symbol
    */
   public String getSymbol();
   
   /**
    * Return the check digit position inside the sequence or -1 if 
    * no check digit is taken from the sequence. 
    * @param sequence Length of integer sequence representing the digits to
    *                 check, not necessary between 0 and 9.
    * @return Position of the check digit.
    */
   public int getCheckDigitPosition(int sequence);
    
}

