/* 
 * THC-Shagg - A modular application to bruteforce check digit algorithms
 * (c) 2001, 2003 Plasmoid <plasmoid@thc.org>, The Hacker's Choice
 * $Id: Modulus10AlgorithmLuhn.java,v 1.6 2003/07/18 10:08:05 plasmoid Exp $ 
 */
package org.thc.shagg.algorithms;

/**
 * This is "my" definition of the Luhn Algorithm. There are different
 * opinions whether the Luhn Algorithm based on the Modulus 10 Algorithm
 * works like the Basic or the Luhn Classes that have been implemented. 
 * <p>
 * The Luhn Algorithm uses a different Addition. Instead of simply adding
 * two elements, the chain sum of the elements is added, e.g. 10+2 = 3 :)
 *
 * @version $Id: Modulus10AlgorithmLuhn.java,v 1.6 2003/07/18 10:08:05 plasmoid Exp $
 * @author Plasmoid (plasmoid@thc.org),
 *                  <a href="http://www.thc.org">THC</a>
 */
public class Modulus10AlgorithmLuhn extends Modulus10Algorithm
{
   /*
    * Init the Modulus 10 Luhn Algorithm.
    */
   public Modulus10AlgorithmLuhn() {
      type="L";
   }

   /**
    * Add method for the Modulus 10 Algorithm. Elements are added by 
    * calculating the chain sum of the multiplied element with the 
    * given weights. 
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @param weights  Integer sequence of the weights to use, may be of
    *                 any length.
    * @param index    Current index
    * @return Sum at index.
    */
   public int mod10Add(int[] sequence, int[] weights, int index) {
      int tmp;
      tmp=sequence[index]*weights[index%weights.length];
      
      /*
       * We are only check for the 2-digit case. If there are more 
       * digits, e.g. 100. We are working on non-numberic-digits and
       * that's not the field of the Luhn Algorithm. IMHO! 
       */    
      if(tmp>9)
         tmp=tmp%10 + (tmp-tmp%10)/10;
      
      return tmp;   
   }

   /**
    * Check method for the Modulus 10 Algorithm. The last digit represents
    * the check digit.
    * @param sequence Integer sequence representing the digits to check,
    *                 not necessary between 0 and 9.
    * @return The check digit.
    */
   public int mod10checkDigit(int[] sequence) {
      return sequence[sequence.length-1];
   }
   
}
